from __future__ import unicode_literals

import json

from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.views.generic import TemplateView

from djrichtextfield import settings


class InitView(TemplateView):
    template_name = 'djrichtextfield/init.js'
    content_type = 'application/javascript'

    def get_settings_json(self):
        return mark_safe(json.dumps(settings.CONFIG['settings'],
                                    default=force_text))

    def get_context_data(self, **kwargs):
        context_data = super(InitView, self).get_context_data(**kwargs)
        context_data['default_settings'] = self.get_settings_json()
        context_data['init_template'] = settings.CONFIG['init_template']
        return context_data



from __future__ import unicode_literals

from django.conf import settings
from django.dispatch import receiver
from django.test.signals import setting_changed

DEFAULT_CONFIG = {
    'js': [],
    'init_template': None,
    'settings': {}
}
CONFIG = DEFAULT_CONFIG.copy()
CONFIG.update(getattr(settings, 'DJRICHTEXTFIELD_CONFIG', {}))


@receiver(setting_changed)
def update_settings(setting=None, value=None, **kwargs):
    global CONFIG
    if setting == 'DJRICHTEXTFIELD_CONFIG':  # pragma: no branch
        CONFIG = DEFAULT_CONFIG.copy()
        if value:  # pragma: no branch
            CONFIG.update(value)



from __future__ import unicode_literals

from django.conf.urls import url

from djrichtextfield.views import InitView

urlpatterns = [
    url('^init.js$', InitView.as_view(), name='djrichtextfield_init')
]



from django.test import TestCase
from django.urls import reverse


class TestUrls(TestCase):
    def test_reverse(self):
        url = reverse('djrichtextfield_init')
        self.assertEqual(url, '/djrichtextfield/init.js')



from __future__ import unicode_literals

from django.db import models

from djrichtextfield.widgets import RichTextWidget


class RichTextField(models.TextField):
    def __init__(self, *args, **kwargs):
        self.field_settings = None
        if 'field_settings' in kwargs:
            self.field_settings = kwargs.pop('field_settings')
        super(RichTextField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = RichTextWidget(field_settings=self.field_settings)
        return super(RichTextField, self).formfield(**kwargs)



import os

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = '--1oh7*5-1_rr%*34^&@e^b59(vgv=zm1wt(wm1ga=pfm73(9i'

DEBUG = True
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'djrichtextfield',
    'testproject.testapp',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'testproject.urls'

WSGI_APPLICATION = 'testproject.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    }
]

STATIC_URL = '/static/'

# Django Rich Text Field
TINYMCE_CONFIG = {
    'js': ['//tinymce.cachefly.net/4.1/tinymce.min.js'],
    'init_template': 'djrichtextfield/init/tinymce.js',
    'settings': {
        'menubar': False,
        'plugins': 'link image table code',
        'toolbar': 'formatselect | bold italic | removeformat |'
                   ' link unlink image table | code',
        'block_formats': 'Paragraph=p;Header 1=h1;Header 2=h2;Header 3=h3',
        'width': 700
    },
    'profiles': {
        'mini': {
            'toolbar': 'bold italic | removeformat'
        }
    }
}

CKEDITOR_CONFIG = {
    'js': ['//cdn.ckeditor.com/4.4.4/standard/ckeditor.js'],
    'init_template': 'djrichtextfield/init/ckeditor.js',
    'settings': {
        'toolbar': [
            {'items': ['Format', '-', 'Bold', 'Italic', '-', 'RemoveFormat']},
            {'items': ['Link', 'Unlink', 'Image', 'Table']},
            {'items': ['Source']}
        ],
        'format_tags': 'p;h1;h2;h3',
        'width': 700,
    },
    'profiles': {
        'mini': {
            'toolbar': [
                {'items': ['Bold', 'Italic', '-', 'RemoveFormat']},
            ]
        }
    }
}

DJRICHTEXTFIELD_CONFIG = CKEDITOR_CONFIG



from django.test import TestCase

from djrichtextfield.models import RichTextField
from djrichtextfield.widgets import RichTextWidget


class TestRichTextField(TestCase):
    def test_formfield_widget(self):
        """
        Model field has RichTextWidget
        """
        self.assertIsInstance(
            RichTextField().formfield().widget, RichTextWidget)

    def test_formfield_widget_passes_settings(self):
        """
        Model field passes setting to widget
        """
        settings = {'foo': True, 'bar': [1, 2, 3]}
        widget = RichTextField(field_settings=settings).formfield().widget
        self.assertEqual(widget.field_settings, settings)



from __future__ import unicode_literals

import json

from django.forms.widgets import Media, Textarea
from django.urls import reverse
from django.utils import six
from django.utils.encoding import force_text
from django.utils.html import format_html

from djrichtextfield import settings


class RichTextWidget(Textarea):
    CSS_CLASS = 'djrichtextfield'
    INIT_URL = 'djrichtextfield_init'
    SETTINGS_ATTR = 'data-field-settings'
    CONTAINER_CLASS = 'field-box'
    PROFILE_KEY = 'profiles'

    def __init__(self, attrs=None, field_settings=None):
        defaults = {'class': self.CSS_CLASS}
        if attrs:
            if 'class' in attrs:
                attrs['class'] = ' '.join([attrs['class'], defaults['class']])
            defaults.update(attrs)
        self.field_settings = field_settings or {}
        super(RichTextWidget, self).__init__(defaults)

    @property
    def media(self):
        js = settings.CONFIG['js']
        js.append(reverse(self.INIT_URL))
        return Media(js=js)

    def get_field_settings(self):
        """
        Get the field settings, if the configured setting is a string try
        to get a 'profile' from the global config.
        """
        field_settings = None
        if self.field_settings:
            if isinstance(self.field_settings, six.string_types):
                profiles = settings.CONFIG.get(self.PROFILE_KEY, {})
                field_settings = profiles.get(self.field_settings)
            else:
                field_settings = self.field_settings
        return field_settings

    def render(self, name, value, attrs=None):
        attrs = attrs or {}
        field_settings = self.get_field_settings()
        if field_settings:
            attrs[self.SETTINGS_ATTR] = json.dumps(field_settings,
                                                   default=force_text)
        textarea = super(RichTextWidget, self).render(name, value, attrs=attrs)
        return format_html(
            '<div class="{0}">{1}</div>', self.CONTAINER_CLASS, textarea)



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from djrichtextfield.models import RichTextField


@python_2_unicode_compatible
class Post(models.Model):
    title = models.CharField(max_length=50)
    lead = RichTextField(field_settings='mini')
    content = RichTextField()

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class Comment(models.Model):
    post = models.ForeignKey(Post, on_delete=models.CASCADE)
    content = models.TextField()

    def __str__(self):
        return 'Comment on "%s"' % self.post.title



import json

from django.test import TestCase
from django.test.utils import override_settings
from django.utils.html import escape

from djrichtextfield.widgets import RichTextWidget

CONFIG = {
    'js': ['foo.js'],
    'settings': {'foo': True, 'bar': [1, 2, 3]},
    'profiles': {
        'simple': {'bar': [1, 2]}
    }
}


class TestRichTextWidget(TestCase):
    def test_css_class(self):
        """
        Has the correct css class set
        """
        self.assertEqual(RichTextWidget().attrs['class'], 'djrichtextfield')

    def test_attrs(self):
        """
        Takes attrs into account
        """
        widget = RichTextWidget({'cols': 10})
        self.assertEqual(widget.attrs['cols'], 10)

    def test_css_class_addition(self):
        """
        Adds to the given css class
        """
        widget = RichTextWidget({'class': 'somethingelse'})
        self.assertEqual(
            widget.attrs['class'], 'somethingelse djrichtextfield')


@override_settings(DJRICHTEXTFIELD_CONFIG=CONFIG)
class SettingsTestCase(TestCase):
    config = CONFIG

    def setUp(self):
        self.widget = RichTextWidget()

    def test_media_js(self):
        """
        Test that the correct javascript files are included.
        """
        self.assertEqual(self.config['js'], self.widget.media._js)

    def test_render(self):
        """
        Test that the rendered textarea is surrounded with a div
        and doesn't include any settings.
        """
        widget = RichTextWidget()
        expected = ('<div class="field-box">'
                    '<textarea class="djrichtextfield" cols="40"'
                    ' name="" rows="10">\r\n</textarea>'
                    '</div>')
        self.assertHTMLEqual(expected, widget.render('', ''))

    def test_render_with_settings(self):
        """
        The field includes the correct data attribute.
        """
        settings = {'foo': False}
        widget = RichTextWidget(field_settings=settings)
        config = json.dumps(settings)
        expected = ('<div class="field-box">'
                    '<textarea class="djrichtextfield" cols="40"'
                    ' data-field-settings="{0}"'
                    ' name="" rows="10">\r\n</textarea>'
                    '</div>'.format(escape(config)))
        self.assertHTMLEqual(expected, widget.render('', ''))

    def test_render_with_profile(self):
        """
        The field includes the correct data attribute.
        """
        widget = RichTextWidget(field_settings='simple')
        config = json.dumps(self.config['profiles']['simple'])
        expected = ('<div class="field-box">'
                    '<textarea class="djrichtextfield" cols="40"'
                    ' data-field-settings="{0}"'
                    ' name="" rows="10">\r\n</textarea>'
                    '</div>'.format(escape(config)))
        self.assertHTMLEqual(expected, widget.render('', ''))

    def test_render_with_missing_profile(self):
        """
        The field gets rendered without a data attribute.
        """
        widget = RichTextWidget(field_settings='missing')
        expected = ('<div class="field-box">'
                    '<textarea class="djrichtextfield" cols="40"'
                    ' name="" rows="10">\r\n</textarea>'
                    '</div>')
        self.assertHTMLEqual(expected, widget.render('', ''))



from django.conf.urls import include, url
from django.contrib import admin

admin.autodiscover()

urlpatterns = [
    url(r'^admin/', admin.site.urls),
    url(r'^djrichtextfield/', include('djrichtextfield.urls'))
]



from django.contrib import admin

from testproject.testapp.forms import CommentForm
from testproject.testapp.models import Comment, Post


class CommentInline(admin.StackedInline):
    model = Comment
    form = CommentForm
    extra = 1


class PostAdmin(admin.ModelAdmin):
    inlines = [CommentInline]


admin.site.register(Post, PostAdmin)



from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse


@override_settings(DJRICHTEXTFIELD_CONFIG={
    'init_template': 'djrichtextfield/init/tinymce.js'
})
class TestInitView(TestCase):
    def setUp(self):
        self.response = self.client.get(reverse('djrichtextfield_init'))

    def test_template(self):
        self.assertEqual(
            self.response.template_name, ['djrichtextfield/init.js'])

    def test_content_type(self):
        self.assertEqual(
            self.response['content-type'], 'application/javascript')

    def test_init_template_in_context(self):
        self.assertEqual('djrichtextfield/init/tinymce.js',
                         self.response.context_data['init_template'])
